/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/**
@fileoverview

This file handles the mozileCopy, mozileCut, and mozilePaste functions,
as well as some other code related to the clipboard.

*/

/********** Clipboard Functionality *************/
/*********Try to cleanout some real bad HTML to make it parseable***********/	
function mozileCleanUpHTML(dirtyHTML) {
	var f = ["mozileCopyCutPaste.js","mozileCleanUpHTML"];
	mozile.bug.write(f,2,"Cleaning HTML...");
	
	try{
		//Basic html4 cleanup:
		re = /<br[^>]*>/gi
		dirtyHTML	=	dirtyHTML.replace(re,"<br />")
		
		re = /<img +[^>]+>/gi									//find <img.......>
		dirtyHTML	=	dirtyHTML.replace(re,"")			//remove <img> tags
		
		re = /&/gi													//find &
		dirtyHTML	=	dirtyHTML.replace(re,"&amp;")	//replace with &amp;
		
		re=/<spacer\w{0,}[^>]*(>)/gi							//custom <spacer tag - CLOSE IT!
		dirtyHTML	=	dirtyHTML.replace(re,"/>")
		
		re=/(<script .*|\s<\/script>)/gi
		dirtyHTML	=	dirtyHTML.replace(re,"")			//remove <script> tags
		
		re=/(<!--.*-->)/gi
		dirtyHTML	=	dirtyHTML.replace(re,"")			//remove comments tags
		
		return dirtyHTML;
		
	}	catch(e) {
			mozile.bug.write(f,1,"An error occured during HTML clean up: "+e);
			return false;
	}
}
	
	
/*********Convert string to a document fragment*************/	
function mozileStrToDOM(xmlStr, namespace, wrapperNodeName) {
	var f = ["mozileCopyCutPaste.js","mozileStrToDOM"];
	mozile.bug.write(f,2,"Converting string to DOM fragment...");
	var p, str;
	
	if(!namespace || typeof(namespace)=="undefined" || namespace=="") {
		if(mozileTarget.document.contentType=="text/html") {
			namespace="http://www.w3.org/TR/REC-html40";
		}
		else {
			namespace	=	"http://www.w3.org/1999/xhtml";							//Default namespace is xhtml
		}
		
	}
		
	if(!wrapperNodeName || typeof(wrapperNodeName)=="undefined" || wrapperNodeName=="")
		wrapperNodeName	=	"removeme"												//Default nodetype to use to wrap up string to create document fragment
	
	try{	
		p	=	new DOMParser()																//Instantiate DOMParser
		try{
			xmlStr = xmlStr.replace(/\s+$|^\s*/gi, "");
			str = "<"+wrapperNodeName+" xmlns=\""+namespace+"\">"+xmlStr+"</"+wrapperNodeName+">"
			var docFrag = p.parseFromString(str,"text/xml")
			
			//Need to insert code that look for parser errors here. Copy that stuff from usenet.
//			mozile.bug.write(f,2,xmlStr +" === "+ docFrag +" ==== "+ printXML(docFrag));
			return docFrag;
		}	catch(e) {
				mozile.bug.write(f,1,"mozileStrToDOM():: Impossible to parse string: "+str+" Error:"+e);
				return false;
		}
	}	catch(e) {
		mozile.bug.write(f,1,"mozileStrToDOM():: Could not create DOMParser object. Error: "+e);
		return false;
	}
}


function mozileInsertHTML(str) {
	var f = ["mozileCopyCutPaste.js","mozileInsertDF"];
	mozile.bug.write(f,2,"Inserting Document Fragment...");
		// take the selection, split the node, and insert the cloned children of the document fragment before the next node
	if(mozileSelection.anchorNode.nodeType == 3) {
		mozileSelection.anchorNode.splitText(mozileSelection.anchorOffset);
	}
	var parent = mozileSelection.anchorNode.parentNode;
	var next = mozileSelection.anchorNode.nextSibling;
	
		// strange hack, there is probably a better way
		// I just create a node, set it's innerHTML, then copy the children of the node
	var node = mozileTarget.document.createElement("wrapper");
	node.innerHTML=str;

	var children = node.childNodes;
	mozile.bug.write(f,2,"Length: "+ children.length + node.nodeName + node.childNodes.length);
	
		// clone each child node "deep" and insert it before "next"
	for(var i=0;i<children.length;i++) {
		var newNode = children[i].cloneNode(true);
		parent.insertBefore(newNode, next);
	}
}

/*  
		// This chunk of code works for HTML pasting in HTML documents
	var str="<p><b>Hello!!</b></p>";

		var node = mozileSelection.anchorNode;
		var r = mozileTarget.document.createRange();
		r.selectNodeContents(node);
		r.deleteContents();
		var df = r.createContextualFragment(html);
			mozile.bug.write(f,2,"DF: "+df +" ==> "+ printXML(df));
			
			//mozile.bug.write(f,1,"Offset: "+ mozileSelection.anchorNode + mozileSelection.anchorOffset);
			// split the current text node
			mozileSelection.anchorNode.splitText(mozileSelection.anchorOffset);
			
			// insert the document element 			
			var parent = mozileSelection.anchorNode.parentNode;
			var next = mozileSelection.anchorNode.nextSibling;
			parent.insertBefore(df,next);
*/				


// Mozile Insert Document Fragment

function mozileInsertDF(df) {
	var f = ["mozileCopyCutPaste.js","mozileInsertDF"];
	mozile.bug.write(f,2,"Inserting Document Fragment...");
		// take the selection, split the node, and insert the cloned children of the document fragment before the next node
	if(mozileSelection.anchorNode.nodeType == 3) {
		mozileSelection.anchorNode.splitText(mozileSelection.anchorOffset);
	}
	var parent = mozileSelection.anchorNode.parentNode;
	var next = mozileSelection.anchorNode.nextSibling;
	
	var children = df.documentElement.childNodes;
	mozile.bug.write(f,2,"Length: "+ children.length + df.documentElement.nodeName + df.documentElement.childNodes.length);
	
		// clone each child node "deep" and insert it before "next"
	for(var i=0;i<children.length;i++) {
		var newNode = children[i].cloneNode(true);
		parent.insertBefore(newNode, next);
	}
}

	// need to remove old children, insert new children, and restore mozileSelection
function mozileReplaceNodeWithDF(node, df) {
	var f = ["mozileCopyCutPaste.js","mozileReplaceNodeWithDF"];
	mozile.bug.write(f,2,"Replacing Document Fragment...");

	var r = node.ownerDocument.createRange();
	r.selectNodeContents(node);
	r.deleteContents();
	
	mozile.bug.write(f,2,"Children left: "+ node.childNodes.length);
	
	var children = df.documentElement.childNodes;
	mozile.bug.write(f,2,"Length: "+ children.length);
			// clone each child node "deep" and append it
	for(var i=0;i<children.length;i++) {
		var newNode = children[i].cloneNode(true);
		node.appendChild(newNode);
	}
	mozile.bug.write(f,2,"DF replaced!");
}

/**
Mozile Paste.

Takes the content of the system clipboard and inserts it at the current selection.
Right now only plain text is handled.

To DO: (Max) Pasting text caused problems because the text nodes were being inserted
but no normalizations were carried out, as required by eDOM.
This crashed eDOM when you next came to edit those text nodes.
If you are going to fix the XHTML/XML paste you might want to think about
normalizing the inserted nodes.
*/
function mozilePaste() {
	var f = ["mozileCopyCutPaste.js","mozilePaste"];
	mozile.bug.write(f,2,"Pasting");
	mozileDo("Paste");

	var preferredFlavor = mozileTarget.document.MOZILE_CURRENT_CONFIG.pasteMode;
	switch (preferredFlavor) {
		case "XML" : case "XHTML": case "HTML" : case "text" :
			break;
		default:
			preferredFlavor = "text";
	}
	mozile.bug.write(f,2,"Preferred Flavor:"+preferredFlavor);

	var clipboard = mozilla.getClipboard();
	if(preferredFlavor=="HTML" || preferredFlavor=="XHTML" || preferredFlavor=="XML") {
		try {
			var html	=	clipboard.getData(MozClipboard.HTML_FLAVOR)		//Using the XML_FLAVOR really does not make much sense as it has to be set to "text/html" anyway (otherwise the getData only grabs text)
			//html	=	mozileCleanUpHTML(html)		//Should this be optional? I think not, 'cause of security concerns... Still...
			mozile.bug.write(f,2,"HTML: "+html);

			switch (mozileTarget.document.contentType) {
				case "text/html" :
				case "application/xhtml+xml" :
					mozile.bug.write(f,2,"Pasting in X/HTML Mode");
					mozileInsertHTML(html);
					mozile.bug.write(f,2,"Node inserted?");
					break;
				default :
					mozile.bug.write(f,2,"Pasting in XML Mode");
					var xdoc = mozileStrToDOM(html);
					mozile.bug.write(f,2,"DOM: "+xdoc);			
					mozileInsertDF(xdoc);
					mozile.bug.write(f,2,"Node inserted?");
			}
			mozileUpdateDisplay();
			return true;
		} catch(e) {
			mozile.bug.write(f,1,"Could not get data from clipboard (HTML/XML mode). Error: "+e);
			preferredFlavor = "text";
		}
	}
	if (preferredFlavor == "text") { // text/unicode
		try {		
		mozile.bug.write(f,2,"Pasting in text Mode");
		var text = clipboard.getData(MozClipboard.TEXT_FLAVOR);
		if(text) {
			mozileSelection.insertNode(mozileTarget.document.createTextNode(text));
		}
		}catch(e){
			mozile.bug.write(f,1,"Paste error (text mode). Error: "+e);
		}
	}
	mozileUpdateDisplay();
	return true;
}

/**
Mozile Copy.

Copies the selected text to the system clipboard.
The selection is currently copied as plain text.
*/
function mozileCopy() {
	var f = ["mozileCopyCutPaste.js","mozileCopy"];
	mozile.bug.write(f,2,"Copy");
	var result = false;
	try {
		var cssr = mozileSelection.getEditableRange();
		if(cssr) {
			if (!cssr.collapsed) {
				// data to save - render as text (temporary thing - move to html later)
				var text = cssr.toString();
				var clipboard = mozilla.getClipboard();
				// clipboard.setData(deletedFragment.saveXML(), "text/html"); // go back to this once, paste supports html paste!
				clipboard.setData(text, MozClipboard.TEXT_FLAVOR);
				result = true;
			}
			cssr.detach();
		}
	} catch(e) {
		mozile.bug.write(f,1,"Copy Error: "+e);
	}
	return result;
}

/**
Mozile Cut
Copy the current selection to the system clipboard, then remove it from the document.
Works in plain text.
*/
function mozileCut() {
	var f = ["mozileCopyCutPaste.js","mozileCut"];
	mozile.bug.write(f,2,"Cutting");
	mozileDo("Cut");
	try {
		var cssr = mozileSelection.getEditableRange();
		if(!cssr) // not an editable area or nothing selected
			return;
		if (!cssr.collapsed) {
			var clipboard = mozilla.getClipboard();
			// data to save - render as text (temporary thing)
			var text = cssr.toString();
			//var deletedFragment = cssr.extractContentsByCSS();
			var objXMLSerializer = new XMLSerializer;
			var deletedFragment = objXMLSerializer.serializeToString(cssr.extractContentsByCSS());
			mozile.bug.write(f,2,"Deleted fragment: "+deletedFragment);
			clipboard.setData(deletedFragment, MozClipboard.HTML_FLAVOR); // go back to this once, paste supports html paste!
			clipboard.setData(text, MozClipboard.TEXT_FLAVOR);
			mozileSelection.selectEditableRange(cssr);
			mozileUpdateDisplay();
		}
		cssr.detach();
	} catch(e) {
		mozile.bug.write(f,1,"Cut Error: "+e);
	}
}

/* You can use this for debugging. */
/*
function printXML(XML) {
	var objXMLSerializer = new XMLSerializer;
	var output = objXMLSerializer.serializeToString(XML);
		// fix stupid uppercase tags!
	output = output.replace( /<(\/?)([A-Z]+)/g,
		function (output,p1,p2,offset,s) {
			return '<'+p1+p2.toLowerCase() ;
		} ) ;
	return output;
}
*/

